#ifndef __MASK_H__
#define __MASK_H__

typedef struct mask mask_t;

// Constructor/destructor
mask_t *mask_create(size_t n);
mask_t *mask_resize(mask_t *, size_t n);
mask_t *mask_copy(mask_t *);
void mask_destroy(mask_t *);

size_t mask_size(mask_t *);

// Elements
bool mask_get(mask_t *, int i);
void mask_set(mask_t *, int i, bool v);

// Operations
void mask_not(mask_t *);
void mask_and(mask_t *, mask_t *);
void mask_or(mask_t *, mask_t *);

size_t mask_count_true(mask_t *);

/**
 * Find the indices of elements in the mask.
 *
 * @arg i Output argument for the indices.  Must be of size returned by
 *        mask_count_true
 */
void mask_nonzero_indices(mask_t *, int *i);

// I/O
void mask_print(mask_t *);

/**
 * Map the indices of a mask to be within the subset.
 *
 * In the process, considers only the intersection of the two input masks.
 */
mask_t *mask_subset_indices(mask_t *subset, mask_t *mask);

/**
 * Map the indices of a mask to be within the superset.
 */
mask_t *mask_superset_indices(mask_t *subset, mask_t *mask);

#endif // #ifndef __MASK_H__
