#ifndef __MATLAB_GPU_H__
#define __MATLAB_GPU_H__

#include <mex.h>

#include <boxopt/boxqp.h>

/**
 * Convert an mxArray to a flat array on the GPU.
 *
 * @note Caller must call @c cudaFree on result
 */
double *matlab_vector_to_gpu(const mxArray *src);
mxArray *matlab_vector_from_gpu(int n, const double *src);

/**
 * @note Caller must call @c cudaFree on the @i property of the result
 */
gpu_mat_t matlab_matrix_to_gpu(const mxArray *src, bool transpose);

gpu_boxqp_t boxqp_gpu_from_matlab(const mxArray *Q, const mxArray *c, const mxArray *l, const mxArray *u);

void boxqp_gpu_cleanup(gpu_boxqp_t *boxqp);

gpfr_options_t matlab_struct_to_gpfr_opts(const mxArray *matopts);
mxArray *matlab_struct_from_optstatus(const boxopt_optstatus_t *status);

/********************** Specific Functions **********************/

void matlab_boxqp(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]);
void matlab_rwprior(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]);
void matlab_rwcoseg(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]);

#endif /* #ifndef __MATLAB_GPU_H__ */
