import sys, os, os.path
from SCons.Script import *

def generate(env):
    # Set environment variables
    env['NVCC'] = 'nvcc'
    env['NVCCFLAGS'] = ''

    env['NVCCCMD'] = '$NVCC -o $TARGET -c $NVCCFLAGS $_CCCOMCOM $SOURCES'
    env['SHNVCCCMD'] = '$NVCC -o $TARGET -c --shared -Xcompiler -fPIC $NVCCFLAGS $_CCCOMCOM $SOURCES'

    if 'CUDA_ROOT' not in env:
        cuda_root = None
        for path in env['ENV']['PATH'].split(':'):
            if os.path.exists(os.path.join(path, env['NVCC'])):
                cuda_root, bin = os.path.split(path)
                assert(bin == 'bin')
                break

        if cuda_root == None:
            print 'Error: Could not find CUDA root directory'
            Exit(1)

    # TODO: use SConf for this

    env['CUDA_ROOT'] = cuda_root
    env.Append(CPPPATH = os.path.join('$CUDA_ROOT', 'include'))

    if os.path.exists(os.path.join(cuda_root, 'lib64')):
        env.Append(LIBPATH = os.path.join('$CUDA_ROOT', 'lib64'))
    else:
        env.Append(LIBPATH = os.path.join('$CUDA_ROOT', 'lib'))

    # Add object builder
    staticObjBuilder, sharedObjBuilder = SCons.Tool.createObjBuilders(env)
    staticObjBuilder.add_action('.cu', '$NVCCCMD')
    staticObjBuilder.add_emitter('.cu', SCons.Defaults.StaticObjectEmitter)
    sharedObjBuilder.add_action('.cu', '$SHNVCCCMD')
    sharedObjBuilder.add_emitter('.cu', SCons.Defaults.SharedObjectEmitter)
    #env.Append(BUILDERS = {'NVCC': staticObjBuilder})

    # Add scanner for .cu files
    SCons.Tool.SourceFileScanner.add_scanner('.cu', SCons.Scanner.C.CScanner())

    # Add libraries
    env.Append(LIBS = ['cudart'])


def exists(env):
    return env.Detect('nvcc')
