# Loosely based on GCH Builder by Tim Blechmann,
# http://www.scons.org/wiki/GchBuilder?action=recall&rev=6

import os, os.path
import shlex

from SCons.Script import *

# TODO: TIP: GCC supports multiple precompiled headers (with different compiler options) by putting them in a folder named "headername.gch" with different names. The builder could use this to create a pch for each set of options used... 

C_HEADER_SUFFIXES = set(['.h'])
CXX_HEADER_SUFFIXES = set(['.hh', '.H', '.hpp', '.hxx', '.h++'])
C_SOURCE_SUFFIXES = set(['.c'])
CXX_SOURCE_SUFFIXES = set(['.cc', '.C', '.cpp', '.cxx', '.c++'])

def command_factory(shared):
    def command(source, target, env, for_signature):
        source = source[0]
        target = target[0]

        if source.get_suffix() in C_HEADER_SUFFIXES:
            if shared:
                cmd = env['SHCCCOM']
                cmd = cmd.replace('$SHCFLAGS', '$SHCPCHFLAGS')
            else:
                cmd = env['CCCOM']
                cmd = cmd.replace('$CFLAGS', '$CPCHFLAGS')

        elif source.get_suffix() in CXX_HEADER_SUFFIXES:
            if shared:
                cmd = env['SHCXXCOM']
                cmd = cmd.replace('$SHCXXFLAGS', '$SHCXXPCHFLAGS')
            else:
                cmd = env['CXXCOM']
                cmd = cmd.replace('$CXXFLAGS', '$CXXPCHFLAGS')

        return cmd

    return command

def suffix(env, sources):
    return sources[0].get_suffix() + '.gch'

# Create emitter to append onto C
def emitter_factory(shared):
    def emitter(target, source, env):
        #if shared:
        #    SCons.Defaults.SharedObjectEmitter(target, source, env)
        #else:
        #    SCons.Defaults.StaticObjectEmitter(target, source, env)

        # Perform standard C scanning
        scanner = env['BUILDERS']['StaticObject'].source_scanner
        path = scanner.path(env)
        deps = scanner(source[0], env, path)

        # TODO: breaks sconsign...?
        #deps = []

        # Add dependency on precompiled header
        deps = [str(f) for f in deps]

        if env.has_key(gchkey):
            for gch in env[gchkey]:
                if str(gch.sources[0]) in deps:
                    env.Depends(target, gch)

        return (target, source)

    return emitter

def generate(env):
    # Add precompiled header builders
    builder = env.Builder(generator = command_factory(False),
                          suffix = suffix,
                          single_source = True)
    shared_builder = env.Builder(generator = command_factory(True),
                                 suffix = suffix,
                                 single_source = True)

    env.Append(BUILDERS = {'PrecompiledHeader': builder})
    env.Append(BUILDERS = {'SharedPrecompiledHeader': shared_builder})

    # Add PCHflags
    env['SHCPCHFLAGS'] = ['$SHCFLAGS']
    env['CPCHFLAGS'] = ['$CFLAGS']
    env['SHCXXPCHFLAGS'] = ['$SHCXXFLAGS']
    env['CXXPCHFLAGS'] = ['$CXXFLAGS']

    # Add new emitters for C source files
    for suf in C_SOURCE_SUFFIXES.union(CXX_SOURCE_SUFFIXES):
        env['BUILDERS']['StaticObject'].add_emitter(suf, emitter_factory(False))
        env['BUILDERS']['SharedObject'].add_emitter(suf, emitter_factory(True))

def exists(env):
    return env.Detect('gcc') or env.Detect('g++')
