from SCons.Script import *
import os, os.path, subprocess

"""Adds a method to the environment to pass commands to a matlab process."""


class MatlabSession:
    def __init__(self, env, cwd = '.'):
        devnull = file(os.devnull, 'w')
        cmd = [env['MATLAB'], '-nojvm', '-nosplash', '-nodisplay']
        self.proc = subprocess.Popen(cmd, stdin = subprocess.PIPE, cwd = cwd)

        for path in env['MATLABPATH']:
            self.addpath(path)

    def __call__(self, cmd):
        print cmd
        self.proc.stdin.write(cmd + '\n')

    def __del__(self):
        self('exit')
        self.proc.wait()

    def addPath(self, path, recursive = False):
        if recursive:
            self('addpath(genpath(\'%s\'))' % str(path))
        else:
            self('addpath(\'%s\')' % str(path))

def generate(env):
    if 'MATLAB' not in env:
        env['MATLAB'] = 'matlab'

    if 'MATLABPATH' not in env:
        env['MATLABPATH'] = []

    def makeSession(env, *args, **kwargs):
        return MatlabSession(env, *args, **kwargs)

    env.AddMethod(makeSession, 'MatlabSession')

def exists(env):
    if 'MATLAB' in env:
        return env.Detect(env['MATLAB'])
    else:
        return env.Detect('matlab')
