from SCons.Script import *
import os, os.path, glob

globmod = glob

def splitext(node):
    return os.path.splitext(node.path)

class NodeSet:
    def __init__(self, env, contents = []):
        self.env = env
        self.nodes = set([])

        self += contents

    def add(self, f):
        if type(f) == str:
            # TODO: handle directories?
            self.nodes.add(self.env.File(f))
        elif type(f) == list:
            for g in f:
                self += g
        elif 'nodes' in vars(f):
            self.nodes.update(f.nodes)
        else:
            self.nodes.add(f)

    def addRecursive(self, dir, glob = '*'):
        for walkpath, walkdirs, walkfiles in os.walk(dir):
            # Ignore hidden directories
            for d in walkdirs:
                if d[0] == '.':
                    walkdirs.remove(d)

            # Select out files with glob
            walkfiles = globmod.glob(os.path.join(walkpath, glob))

            # Add to list
            self += walkfiles

    def __iadd__(self, f):
        self.add(f)
        return self

    def __call__(self):
        return list(self.nodes)

    def __iter__(self):
        return self.nodes.__iter__()

    def lineCount(self):
        n = 0
        for f in self.nodes:
            for l in file(f.abspath).readlines():
                n += 1

        return n

    def extensionFilter(self, select):
        ls = []

        if type(select) == str:
            select = [select]

        for n in self.nodes:
            if splitext(n)[1] in select:
                ls.append(n)

        return ls

    def extensionMap(self, mapping):
        ls = []

        for n in self.nodes:
            base,ext = os.path.splitext(n.abspath)
            
            if ext in mapping:
                ls.append(self.env.File(base + mapping[ext]))

        return ls

    def CppSources(self):
        return self.extensionFilter(['.cc', '.cpp', '.C'])

    def CppHeaders(self):
        return self.extensionFilter(['.h', '.hpp', '.hh', '.H'])

    def CSources(self):
        return self.extensionFilter('.c')

    def CHeaders(self):
        return self.extensionFilter('.h')

    def ObjectFiles(self):
        osuff = self.env['OBJSUFFIX']
        objs = self.extensionFilter(osuff)

        extmap = {}
        for ext in ['.c', '.cc', '.cpp', '.C', '.cu']:
            extmap[ext] = osuff
        objs.extend(self.extensionMap(extsuff))

        return objs

    def CUDASources(self):
        return self.extensionFilter('.cu')

    def SharedObjectFiles(self):
        objs = self.extensionFilter(self.env['SHOBJSUFFIX'])
        objs.extend(self.extensionMap({'.c': self.env['SHOBJSUFFIX']}))

        return objs

def generate(env):
    env['SOURCE_LISTS'] = {}

    # Add method
    def get(env, key):
        if key not in env['SOURCE_LISTS']:
            env['SOURCE_LISTS'][key] = NodeSet(env)

        return env['SOURCE_LISTS'][key]
    env.AddMethod(get, 'NodeSet')

def exists(env):
    True
