#ifndef __GPU_BOXQP_H__
#define __GPU_BOXQP_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <stdbool.h>
#include "grad_proj.h"
#include "gpfr.h"

#if !defined(CUSPARSE_H_)
struct cusparseContext;
typedef struct cusparseContext *cusparseHandle_t;
#endif

typedef enum {
  GPU_MAT_DENSE,
  GPU_MAT_COO,
  GPU_MAT_CSC,
  GPU_MAT_CSR

} gpu_mat_format_t;

typedef struct {
  gpu_mat_format_t format;

  int m, n, nnz;

  int *i, *j;
  double *v;

} gpu_mat_t;

typedef struct {
  gpu_mat_t Q;
  double *c;

  double *l, *u;
} gpu_boxqp_t;

gpu_mat_t gpu_mat_transpose(cusparseHandle_t handle, const gpu_mat_t *mat);
void gpu_mat_free(gpu_mat_t *mat);

/**
 * Solve the bound-constrained QP
 *
 *   min_x   1/2 * x' * Q * x + c' * x
 *   s.t.    x <= l < = u
 *
 * Given a function @c Qmv which performs the matrix-vector multiplication
 * Q * x.
 *
 * @todo Equation formatting above.
 */
boxopt_optstatus_t gpu_boxqp_grad_proj(int n, double *x, 
  void *user_data, void (*Qmv)(void *user_data, const double *x, double *Qx),
  const double *c, const double *l, const double *u,
  const gpfr_options_t *opts);

boxopt_optstatus_t gpu_boxqp_matrix_grad_projf(double *x, const gpu_boxqp_t *QP, const gpfr_options_t *opts, cusparseHandle_t cusparse_handle);
           
#ifdef __cplusplus
}
#endif

#endif /* #ifndef __GPU_BOXQP_H__ */
