#include "debug.h"

#include <stdlib.h>
#include <stdio.h>
#include <cuda_runtime.h>

void print_set(int n, const bool *x, int group, const char *spacing)
{
  int i;

  if(spacing == NULL) {
    spacing = "";
  }

  for(i = 0; i < n; i++) {
    if(group > 0 && i != 0 && i % group == 0) {
      fputc(' ', stdout);
    }

    printf("%c", x[i] ? 't' : 'f');

    fputs(spacing, stdout);
  }
  fputc('\n', stdout);
}

void print_set_gpu(int n, const bool *x, int group, const char *spacing)
{
  bool *cpu;

  cpu = malloc(n * sizeof(bool));
  cudaMemcpy(cpu, x, n * sizeof(bool), cudaMemcpyDeviceToHost);
  print_set(n, cpu, group, spacing);
  free(cpu);
}

void print_vector(int n, const double *x)
{
  int i;

  fputc('(', stdout);
  for(i = 0; i < n; i++) {
    if(i != 0) {
      fputc(',', stdout); fputc(' ', stdout);
    }

    printf("%e", (float)(x[i]));
  }
  fputc(')', stdout);
  fputc('\n', stdout);
}

void print_vector_gpu(int n, const double *x)
{
  double *cpu;

  cpu = malloc(n * sizeof(double));
  cudaMemcpy(cpu, x, n * sizeof(double), cudaMemcpyDeviceToHost);
  print_vector(n, cpu);
  free(cpu);
}

void print_ints(int n, const int *v)
{
  int i;

  for(i = 0; i < n; i++) {
    if(i != 0) {
      fputc(',', stdout); fputc(' ', stdout);
    }

    printf("%d", v[i]);
  }
  fputc('\n', stdout);
}

void print_ints_gpu(int n, const int *v)
{
  int *cpu;

  cpu = malloc(n * sizeof(int));
  cudaMemcpy(cpu, v, n * sizeof(int), cudaMemcpyDeviceToHost);
  print_ints(n, cpu);
  free(cpu);
}
