#ifndef __GPU_DEBUG_H__
#define __GPU_DEBUG_H__

#include <stdlib.h>
#include <stdbool.h>
#include <stdio.h>

#include <cuda_runtime.h>
#include <cusparse.h>

#ifdef __cplusplus
extern "C" {
#endif

void print_set(int n, const bool *x, int group, const char *spacing);
void print_set_gpu(int n, const bool *x, int group, const char *spacing);

void print_vector(int n, const double *x);
void print_vector_gpu(int n, const double *x);

void print_ints(int n, const int *v);
void print_ints_gpu(int n, const int *v);

static inline void cudaAssertSuccess(cudaError_t err)
{
  if(err != cudaSuccess) {
    printf("Error: %s\n", cudaGetErrorString(err));
  }
  exit(1);
}

static inline void cusparseAssertSuccess(cusparseStatus_t s)
{
  if(s != CUSPARSE_STATUS_SUCCESS) {
    printf("Error: cusparse status %d\n", (int)s);
  }
  exit(1);
}

#ifdef __cplusplus
}
#endif


#endif // #ifndef __GPU_DEBUG_H__
