#ifndef __BOXOPT_GPFR_H__
#define __BOXOPT_GPFR_H__

#include "grad_proj.h"
#include "optstatus.h"

#ifdef __cplusplus
extern "C" {
#endif


typedef struct {
  double grad_norm;
  int max_iters;

  double eta1;
  double eta2;

  double obj_const;
  int print_interval;
  void (*printf)(const char *);

} gpfr_options_t;

extern const gpfr_options_t gpfr_default_options;

/**
 * Solve a bound-constrained problem with the GPFR algorithm.
 *
 * Gradient Projection/Fletcher-Reeves (GPFR) alternates between the
 * gradient projection and projected Fletcher-Reeves descent methods.
 *
 * @note Arguments must point to CUDA device memory.
 * @todo citation
 */
boxopt_optstatus_t boxopt_gpu_gpfr(
  int n, double *x, const double *l, const double *u,
  const gpu_functionf_t *f, const gpfr_options_t *opts);

#ifdef __cplusplus
}
#endif

#endif /* #ifndef __BOXOPT_GPFR_H__ */
