#ifndef __GRAD_PROJ_H__
#define __GRAD_PROJ_H__

#include "optstatus.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @note Arguments are pointers to CUDA device memory.
 */
typedef struct {
  double (*objective)(void *user_data, int n, const double *x);
  void (*gradient)(void *user_data, int n, const double *x, double *g);

  double (*directional_derivative2)(void *user_data, int n, const double *x, const double *d);

  void *user_data;

} gpu_functionf_t;

typedef struct {
  double grad_norm;

  int max_iters;
} grad_proj_options_t;

extern const grad_proj_options_t grad_proj_default_options;

/**
 * @note Arguments must point to CUDA device memory.
 */
boxopt_optstatus_t gpu_grad_projf(
  int n, double *x, const double *l, const double *u,
  const gpu_functionf_t *f, const grad_proj_options_t *opts);

#ifdef __cplusplus
}
#endif

#endif /* #ifndef __GRAD_PROJ_H__ */
