#include "convert.h"

#define CAST(x) make_cuFloatComplex(x, 0)
#define FUNC gpu_StoC
#define SRC_ELT float
#define DST_ELT cuComplex
#include "cast_kernel.h"

#define CAST(x) (double)x
#define FUNC gpu_StoD
#define SRC_ELT float
#define DST_ELT double
#include "cast_kernel.h"

#define CAST(x) make_cuDoubleComplex((double)x, 0)
#define FUNC gpu_StoZ
#define SRC_ELT float
#define DST_ELT cuDoubleComplex
#include "cast_kernel.h"

#define CAST(x) cuComplexFloatToDouble(x)
#define FUNC gpu_CtoZ
#define SRC_ELT cuComplex
#define DST_ELT cuDoubleComplex
#include "cast_kernel.h"

#define CAST(x) (float)x
#define FUNC gpu_DtoS
#define SRC_ELT double
#define DST_ELT float
#include "cast_kernel.h"

#define CAST(x) make_cuFloatComplex((float)x, 0)
#define FUNC gpu_DtoC
#define SRC_ELT double
#define DST_ELT cuComplex
#include "cast_kernel.h"

#define CAST(x) make_cuDoubleComplex((float)x, 0)
#define FUNC gpu_DtoZ
#define SRC_ELT double
#define DST_ELT cuDoubleComplex
#include "cast_kernel.h"

#define CAST(x) cuComplexDoubleToFloat(x)
#define FUNC gpu_ZtoC
#define SRC_ELT cuDoubleComplex
#define DST_ELT cuComplex
#include "cast_kernel.h"


size_t gpu_type_length(int count, linalg_elt_type_t type)
{
  size_t elt_size;

  switch(type) {
  case LINALG_SINGLE:
    elt_size = sizeof(float);
    break;
  case LINALG_DOUBLE:
    elt_size = sizeof(double);
    break;
  case LINALG_COMPLEX:
    elt_size = sizeof(cuComplex);
    break;
  case LINALG_DOUBLE_COMPLEX:
    elt_size = sizeof(cuDoubleComplex);
    break;
  default:
    elt_size = 0;
    break;
  }

  return elt_size * count;
}

linalg_status_t gpu_convert(int n, linalg_elt_type_t dst_type, void *dst, linalg_elt_type_t src_type, const void *src)
{
  switch(src_type) {
  case LINALG_SINGLE:
    switch(dst_type) {
    case LINALG_SINGLE:
      cudaMemcpy(dst, src,  n * sizeof(float), cudaMemcpyDeviceToDevice);
      break;
    case LINALG_DOUBLE:
      gpu_StoD(n, (double *)dst, (const float *)src);
      break;
    case LINALG_COMPLEX:
      gpu_StoC(n, (cuComplex *)dst, (const float *)src);
      break;
    case LINALG_DOUBLE_COMPLEX:
      gpu_StoZ(n, (cuDoubleComplex *)dst, (const float *)src);
      break;
    default:
      return LINALG_OPERATION_NOT_SUPPORTED;
    }
    break;

  case LINALG_DOUBLE:
    switch(dst_type) {
    case LINALG_SINGLE:
      gpu_DtoS(n, (float *)dst, (const double *)src);
      break;
    case LINALG_DOUBLE:
      cudaMemcpy(dst, src, n * sizeof(double), cudaMemcpyDeviceToDevice);
      break;
    case LINALG_COMPLEX:
      gpu_DtoC(n, (cuComplex *)dst, (const double *)src);
      break;
    case LINALG_DOUBLE_COMPLEX:
      gpu_DtoZ(n, (cuDoubleComplex *)dst, (const double *)src);
      break;
    default:
      return LINALG_OPERATION_NOT_SUPPORTED;
    }
    break;

  case LINALG_COMPLEX:
    switch(dst_type) {
    case LINALG_COMPLEX:
      cudaMemcpy(dst, src, n * sizeof(cuComplex), cudaMemcpyDeviceToDevice);
      break;
    case LINALG_DOUBLE_COMPLEX:
      gpu_CtoZ(n, (cuDoubleComplex *)dst, (const cuComplex *)src);
      break;
    default:
      return LINALG_OPERATION_NOT_SUPPORTED;
    }
    break;

  case LINALG_DOUBLE_COMPLEX:
    switch(dst_type) {
    case LINALG_COMPLEX:
      gpu_ZtoC(n, (cuComplex *)dst, (const cuDoubleComplex *)src);
      break;
    case LINALG_DOUBLE_COMPLEX:
      cudaMemcpy(dst, src, n * sizeof(cuDoubleComplex), cudaMemcpyDeviceToDevice);
      break;
    default:
      return LINALG_OPERATION_NOT_SUPPORTED;
    }
    break;

  default:
    return LINALG_OPERATION_NOT_SUPPORTED;
  }

  return LINALG_OK;
}
