#include "matrix_internal.h"

#include <stdlib.h>
#include <assert.h>

matrix_t *matrix_clone(const matrix_t *A)
{
	assert(A->v->clone != NULL);
	return A->v->clone(A);
}

void matrix_destroy(matrix_t *A)
{
  if(A == NULL) {
    return;
  }

	assert(A->v->destroy != NULL);
	A->v->destroy(A);
}

// Dimensions
int matrix_ndims(const matrix_t *A)
{
	return A->ndims;
}

const int *matrix_dims(const matrix_t *A)
{
	return A->dims;
}

// CUDA matrices
bool matrix_gpu(const matrix_t *A)
{
	return A->gpu;
}

matrix_t *matrix_from_gpu(const matrix_t *A)
{
	if(A->v->from_gpu == NULL) {
		return NULL;
	}
	return A->v->from_gpu(A);
}

matrix_t *matrix_to_gpu(const matrix_t *A)
{
	if(A->v->to_gpu == NULL) {
		return NULL;
	}
	return A->v->to_gpu(A);
}

matrix_storage_format_t matrix_storage_format(const matrix_t *A)
{
	return A->format;
}

matrix_t *matrix_storage_to_format(const matrix_t *A, matrix_storage_format_t fmt)
{
	matrix_t *(*convert)(const matrix_t *A);

	// Select conversion function
	switch(fmt) {
	case MATRIX_FULL:
		convert = A->v->to_full;
	case MATRIX_CSC:
		convert = A->v->to_csc;
	case MATRIX_CSR:
		convert = A->v->to_csr;
 	case MATRIX_COO:
		convert = A->v->to_coo;
	}

	if(convert == NULL) {
		return NULL;
	}
	return convert(A);
}


bool matrix_storage_is_sparse(matrix_storage_format_t fmt)
{
	switch(fmt) {
	case MATRIX_FULL:
    return false;
	case MATRIX_CSC:
		return true;
	case MATRIX_CSR:
		return true;
 	case MATRIX_COO:
		return true;
	}
}

// Element types
linalg_elt_type_t matrix_elt_type(const matrix_t *A)
{
	return A->type;
}

matrix_t *matrix_to_elt_type(const matrix_t *A, linalg_elt_type_t type)
{
	if(A->v->to_elt_type == NULL) {
		return NULL;
	}
	return A->v->to_elt_type(A, type);
}
