#ifndef __LINALG_MATRIX_H__
#define __LINALG_MATRIX_H__

#include "elt.h"
#include "vector.h"

typedef enum matrix_storage_format matrix_storage_format_t;
typedef struct matrix matrix_t;

// Constructors/destructor
matrix_t *matrix_create_full(linalg_elt_type_t type, int ndims, const int *dims);
matrix_t *matrix_create_csc(linalg_elt_type_t type, int rows, int cols);
matrix_t *matrix_create_csr(linalg_elt_type_t type, int rows, int cols);
matrix_t *matrix_create_coo(linalg_elt_type_t type, int ndims, const int *dims);

matrix_t *matrix_clone(const matrix_t *A);
void matrix_destroy(matrix_t *A);

// Dimensions
int matrix_ndims(const matrix_t *A);
const int *matrix_dims(const matrix_t *A);

// CUDA matrices
bool matrix_gpu(const matrix_t *A);
matrix_t *matrix_from_gpu(const matrix_t *A);
matrix_t *matrix_to_gpu(const matrix_t *A);

// Storage format
enum matrix_storage_format {
	MATRIX_FULL = 0,

	MATRIX_CSC = 1,
	MATRIX_CSR = 2,
 	MATRIX_COO = 3

};

matrix_storage_format_t matrix_storage_format(const matrix_t *A);
matrix_t *matrix_storage_to_format(const matrix_t *A, matrix_storage_format_t fmt);

bool matrix_storage_is_sparse(matrix_storage_format_t fmt);

// Element types
linalg_elt_type_t matrix_elt_type(const matrix_t *A);
matrix_t *matrix_to_elt_type(const matrix_t *A, linalg_elt_type_t type);

// Nonzeros
int matrix_nnz(const matrix_t *A);

#endif // #ifndef __LINALG_MATRIX_H__
