#include "matrix_description_internal.h"

#include <stdlib.h>

/////////////////////////// Constructors/Destructors //////////////////

matdesc_t *matdesc_create()
{
  matdesc_t *D;

  D = (matdesc_t *)malloc(sizeof(matdesc_t));
  matdesc_init(D);

  return D;
}

void matdesc_init(matdesc_t *D)
{
  D->computation_unit = LINALG_PLAIN;
  D->storage = MATRIX_FULL;
  D->elt_type = LINALG_DOUBLE;

  D->bit_field = BIT_FIELD_DEFAULT;
}

matdesc_t *matdesc_clone(const matdesc_t *src)
{
  matdesc_t *dst;

  dst = (matdesc_t *)malloc(sizeof(matdesc_t));
  *dst = *src;

  return dst;
}

void matdesc_assign(matdesc_t *dst, const matdesc_t *src)
{
  *dst = *src;
}

void matdesc_deinit(matdesc_t *D)
{
}

void matdesc_destroy(matdesc_t *D)
{
  free(D);
}

/////////////////////////// Accessors/Mutators ////////////////////////

void matdesc_computation_unit_set(matdesc_t *D, linalg_computation_unit_t cu)
{
  D->computation_unit = cu;
}

linalg_computation_unit_t matdesc_computation_unit_get(const matdesc_t *D)
{
  return D->computation_unit;
}


void matdesc_set_storage_scheme(matdesc_t *D, matrix_storage_scheme_t ss)
{
  D->storage = ss;
}

matrix_storage_scheme_t matdesc_get_storage_scheme(const matdesc_t *D)
{
  return D->storage;
}

bool matrix_storage_scheme_is_sparse(matrix_storage_scheme_t ss)
{
  return ss == MATRIX_FULL;
}

void matdesc_set_elt_type(matdesc_t *D, linalg_elt_type_t et)
{
  D->elt_type = et;
}

linalg_elt_type_t matdesc_get_elt_type(const matdesc_t *D)
{
  return D->elt_type;
}

void matdesc_set_symmetric(matdesc_t *D, bool b)
{
  if(b) {
    D->bit_field |= BIT_SYMMETRIC;
  }
  else {
    D->bit_field &= ~BIT_SYMMETRIC;
  }
}

bool matdesc_get_symmetric(const matdesc_t *D)
{
  return (D->bit_field & BIT_SYMMETRIC) != 0;
}
