#ifndef __LINALG_MATRIX_DESCRIPTION_H__
#define __LINALG_MATRIX_DESCRIPTION_H__

#include <stdbool.h>

typedef struct matdesc matdesc_t;

// Constructors/Destructor
matdesc_t *matdesc_create();
matdesc_t *matdesc_clone(const matdesc_t *D);
void matdesc_assign(matdesc_t *dst, const matdesc_t *src);
void matdesc_destroy(matdesc_t *D);

// Computation unit
typedef enum {
  LINALG_PLAIN = 0,
  LINALG_GPU = 1

} linalg_computation_unit_t;

void matdesc_computation_unit_set(matdesc_t *D, linalg_computation_unit_t);
linalg_computation_unit_t matdesc_computation_unit_get(const matdesc_t *D);

// Storage scheme
typedef enum {
	MATRIX_FULL = 0,

	MATRIX_CSC = 1,
	MATRIX_CSR = 2,
 	MATRIX_COO = 3

} matrix_storage_scheme_t;

void matdesc_set_storage_scheme(matdesc_t *D, matrix_storage_scheme_t);
matrix_storage_scheme_t matdesc_get_storage_scheme(const matdesc_t *D);

bool matrix_storage_scheme_is_sparse(matrix_storage_scheme_t);

// Element type
typedef enum {
  LINALG_SINGLE = 0,
  LINALG_DOUBLE = 1,
  LINALG_COMPLEX = 2,
  LINALG_DOUBLE_COMPLEX = 3,

} linalg_elt_type_t;

void matdesc_set_elt_type(matdesc_t *D, linalg_elt_type_t);
linalg_elt_type_t matdesc_get_elt_type(const matdesc_t *D);

// Matrix properties
void matdesc_set_symmetric(matdesc_t *D, bool);
bool matdesc_get_symmetric(const matdesc_t *D);



#endif // #ifndef __LINALG_MATRIX_DESCRIPTION_H__
