#include "convert.h"

static inline complex makefloatcomplex(float r, float i)
{
  complex c;

  c.r = r;
  c.i = i;

  return c;
}

#define CAST(x) makefloatcomplex(x, 0)
#define FUNC plain_StoC
#define SRC_ELT float
#define DST_ELT complex
#include "convert_type.h"

#define CAST(x) (double)x
#define FUNC plain_StoD
#define SRC_ELT float
#define DST_ELT double
#include "convert_type.h"

static inline doublecomplex makedoublecomplex(double r, double i)
{
  doublecomplex z;

  z.r = r;
  z.i = i;

  return z;
}

#define CAST(x) makedoublecomplex((double)x, 0)
#define FUNC plain_StoZ
#define SRC_ELT float
#define DST_ELT doublecomplex
#include "convert_type.h"

static inline doublecomplex complexfloat2double(complex c)
{
  doublecomplex z;

  z.r = (double)c.r;
  z.i = (double)c.i;

  return z;
}

#define CAST(x) complexfloat2double(x)
#define FUNC plain_CtoZ
#define SRC_ELT complex
#define DST_ELT doublecomplex
#include "convert_type.h"

#define CAST(x) (float)x
#define FUNC plain_DtoS
#define SRC_ELT double
#define DST_ELT float
#include "convert_type.h"

#define CAST(x) makefloatcomplex((float)x, 0)
#define FUNC plain_DtoC
#define SRC_ELT double
#define DST_ELT complex
#include "convert_type.h"

#define CAST(x) makedoublecomplex((float)x, 0)
#define FUNC plain_DtoZ
#define SRC_ELT double
#define DST_ELT doublecomplex
#include "convert_type.h"

static inline complex complexdouble2float(doublecomplex z)
{
  complex c;

  c.r = (float)z.r;
  c.i = (float)z.i;

  return c;
}

#define CAST(x) complexdouble2float(x)
#define FUNC plain_ZtoC
#define SRC_ELT doublecomplex
#define DST_ELT complex
#include "convert_type.h"

size_t plain_type_length(int count, linalg_elt_type_t type)
{
  size_t elt_size;

  switch(type) {
  case LINALG_SINGLE:
    elt_size = sizeof(float);
    break;
  case LINALG_DOUBLE:
    elt_size = sizeof(double);
    break;
  case LINALG_COMPLEX:
    elt_size = sizeof(complex);
    break;
  case LINALG_DOUBLE_COMPLEX:
    elt_size = sizeof(doublecomplex);
    break;
  default:
    elt_size = 0;
    break;
  }

  return elt_size * count;
}

linalg_status_t plain_convert(int n, linalg_elt_type_t dst_type, void *dst, linalg_elt_type_t src_type, const void *src)
{
  switch(src_type) {
  case LINALG_SINGLE:
    switch(dst_type) {
    case LINALG_SINGLE:
      memcpy(dst, src,  n * sizeof(float));
      break;
    case LINALG_DOUBLE:
      plain_StoD(n, (double *)dst, (const float *)src);
      break;
    case LINALG_COMPLEX:
      plain_StoC(n, (complex *)dst, (const float *)src);
      break;
    case LINALG_DOUBLE_COMPLEX:
      plain_StoZ(n, (doublecomplex *)dst, (const float *)src);
      break;
    default:
      return LINALG_OPERATION_NOT_SUPPORTED;
    }
    break;

  case LINALG_DOUBLE:
    switch(dst_type) {
    case LINALG_SINGLE:
      plain_DtoS(n, (float *)dst, (const double *)src);
      break;
    case LINALG_DOUBLE:
      memcpy(dst, src, n * sizeof(double));
      break;
    case LINALG_COMPLEX:
      plain_DtoC(n, (complex *)dst, (const double *)src);
      break;
    case LINALG_DOUBLE_COMPLEX:
      plain_DtoZ(n, (doublecomplex *)dst, (const double *)src);
      break;
    default:
      return LINALG_OPERATION_NOT_SUPPORTED;
    }
    break;

  case LINALG_COMPLEX:
    switch(dst_type) {
    case LINALG_COMPLEX:
      memcpy(dst, src, n * sizeof(complex));
      break;
    case LINALG_DOUBLE_COMPLEX:
      plain_CtoZ(n, (doublecomplex *)dst, (const complex *)src);
      break;
    default:
      return LINALG_OPERATION_NOT_SUPPORTED;
    }
    break;

  case LINALG_DOUBLE_COMPLEX:
    switch(dst_type) {
    case LINALG_COMPLEX:
      plain_ZtoC(n, (complex *)dst, (const doublecomplex *)src);
      break;
    case LINALG_DOUBLE_COMPLEX:
      memcpy(dst, src, n * sizeof(doublecomplex));
      break;
    default:
      return LINALG_OPERATION_NOT_SUPPORTED;
    }
    break;

  default:
    return LINALG_OPERATION_NOT_SUPPORTED;
  }

  return LINALG_OK;
}
