#ifndef __LINALG_VECTOR_H__
#define __LINALG_VECTOR_H__

#include <stdbool.h>

#include "matrix_description.h"
#include "status.h"

typedef struct {float r, i;} complex;
typedef struct {double r, i;} doublecomplex;

typedef struct vector vector_t;

// Constructors/destructors
vector_t *vector_clone(const vector_t *v);
linalg_status_t vector_destroy(vector_t *v);

linalg_status_t vector_assign(vector_t *dst, const vector_t *src);

linalg_status_t vector_resize(vector_t *v, int new_length);

/**
 * Retrieve the element count of @c v
 */
int vector_length(const vector_t *v);

/**
 * Retrieve the description of the vector.
 */
const matdesc_t *vector_description(const vector_t *v);

/////////////////////////// Norms ////////////////////////////

float vector_norm1f(const vector_t *v);
double vector_norm1(const vector_t *v);

float vector_norm2f(const vector_t *v);
double vector_norm2(const vector_t *v);

float vector_norminftyf(const vector_t *v);
double vector_norminfty(const vector_t *v);

///////////////////////// Max/Min Element ///////////////////////

int vector_iamin(const vector_t *v);
int vector_iamax(const vector_t *v);

///////////////////////// Element Access ////////////////////////

complex vector_celt(const vector_t *v, int i);
doublecomplex vector_zelt(const vector_t *v, int i);

///////////////////////// Vector Math ////////////////////////

complex vector_cdot(const vector_t *u, const vector_t *v);
doublecomplex vector_zdot(const vector_t *u, const vector_t *v);

void vector_sscale(float s, vector_t *v);
void vector_dscale(double s, vector_t *v);
void vector_cscale(complex s, vector_t *v);
void vector_zscale(doublecomplex s, vector_t *v);

void vector_saxpy(float alpha, const vector_t *x, vector_t *y);
void vector_daxpy(double alpha, const vector_t *x, vector_t *y);
void vector_caxpy(complex alpha, const vector_t *x, vector_t *y);
void vector_zaxpy(doublecomplex alpha, const vector_t *x, vector_t *y);

#endif // #ifndef __LINALG_VECTOR_H__
