#ifndef __LINALG_VECTOR_INTERNAL_H__
#define __LINALG_VECTOR_INTERNAL_H__

#include "vector.h"
#include "matrix_description_internal.h"

typedef struct vector_vtable vector_vtable_t;

struct vector
{
  const vector_vtable_t *v;

  int length;
  matdesc_t desc;
};

typedef struct plain_vector plain_vector_t;

struct plain_vector
{
  vector_t base;
  void *data;
};

struct vector_vtable
{
  vector_t *(*clone)(const vector_t *base);
  linalg_status_t (*destroy)(vector_t *base);

  /**
   * An assign operation that only needs to run within the same computation
   * unit.
   *
   * @note Can assume lengths are the same.
   */
  linalg_status_t (*assign)(vector_t *dst, const vector_t *src);

  /**
   * An assign operation to convert to a plain vector.
   *
   * @note Can assume lengths are the same.
   */
  linalg_status_t (*to_plain)(const vector_t *base, plain_vector_t *plain);

  /**
   * An assign operation to convert from a plain vector.
   *
   * @note Can assume lengths are the same.
   */
  linalg_status_t (*from_plain)(vector_t *base, const plain_vector_t *plain);

  /**
   * Change the number of elements allocated in the vector.
   */
  linalg_status_t (*resize)(vector_t *base, int new_length);

  ////////////////////////////// Norms //////////////////////////

  float (*norm1f)(const vector_t *base);
  double (*norm1)(const vector_t *base);

  float (*norm2f)(const vector_t *base);
  double (*norm2)(const vector_t *base);

  ////////////////////////////// Max/Min ///////////////////////////

  int (*iamax)(const vector_t *base);
  int (*iamin)(const vector_t *base);

  //////////////////////////// Element Access //////////////////////
  doublecomplex (*elt)(const vector_t *base, int i);

  //////////////////////////// Vector Math /////////////////////////
  doublecomplex (*dot)(const vector_t *u, const vector_t *v);
  void (*scale)(doublecomplex s, vector_t *v);
  void (*axpy)(doublecomplex alpha, const vector_t *x, vector_t *y);
};

#endif // #ifndef __LINALG_VECTOR_INTERNAL_H__
