#include "mv.h"

#include <stdio.h>

static __global__ void mv_kernel(
  int base, int m, int n,
  const int *Ai, const int *Aj, const double *Av,
  const double *x, double *Ax)
{
  int me, k, row, col;

  // Get which row we're dealing with
  me = blockDim.x * blockIdx.x + threadIdx.x;
  row = me + base;
  if(row >= m) {
    return;
  }

  Ax[row] = 0;
  ///*
  for(k = Ai[row]; k < Ai[row+1]; k++) {
    col = Aj[k];

    Ax[row] += Av[k];//Av[k] * x[col];
  }
  //*/
}

void *cpudup(size_t s, const void *gpu_mem)
{
  void *p;

  p = malloc(s);
  cudaMemcpy(p, gpu_mem, s, cudaMemcpyDeviceToHost);

  return p;
}

void mv(int m, int n, const int *Ai, const int *Aj, const double *Av, const double *x, double *Ax, struct cudaDeviceProp *devprops)
{
  int threads, blocks, stride, base;

  // Get the number of necessary kernel calls
  threads = devprops->maxThreadsPerBlock;
  //threads = 4;
  if(m < threads) {
    threads = m;
    blocks = 1;
  }
  blocks = devprops->maxGridSize[0];
  stride = threads * blocks;
  if(stride > m) {
    blocks = (m + threads - 1) /  threads;
    stride = threads * blocks;
  }

  //Ai = cpudup(m * sizeof(int), Ai);
  //int nnz = Ai[m] - Ai[0];
  //Aj = cpudup(m * sizeof(int), Ai);

  for(base = 0; base < m; base += stride) {
    mv_kernel<<<blocks, threads>>>(base, m, n, Ai, Aj, Av, x, Ax);

    break;
  }
}
