#ifndef __BOXOPT_SET_H__
#define __BOXOPT_SET_H__

#include <stdbool.h>
#include <cuda_runtime.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Calculate the symmetric difference between boolean vectors @c S and @c T.
 */
void set_symm_diff(
  int n, bool *diff, const bool *S, const bool *T,
  struct cudaDeviceProp *devprops);

/**
 * Returns true iff any of the booleans in @c S are true.
 *
 * @arg @c work A vector of bools on the device of length 2*n
 * @arg @c host_work A vector of bools on the host of length n
 */
bool set_or(int n, const bool *S, 
            bool *work, bool *host_work, struct cudaDeviceProp *devprops);

/**
 * Return true iff @c S and @c T are equal.
 *
 * @arg @c work A vector of bools on the device of length 3*n
 * @arg @c host_work A vector of bools on the host of length n
 */
bool sets_equal(int n, const bool *S, const bool *T,
                bool *work, bool *host_work, struct cudaDeviceProp *devprops);

#ifdef __cplusplus
}
#endif

#endif // #ifndef __BOXOPT_SET_H__
