#include <stdio.h>
#include <stdlib.h>
#include <cuda_runtime.h>
#include "cusparse.h"

#include "../src/debug.h"

#define CLEANUP(s)                                   \
do {                                                 \
    printf ("%s\n", s);                              \
    if (yHostPtr)           free(yHostPtr);          \
    if (zHostPtr)           free(zHostPtr);          \
    if (xIndHostPtr)        free(xIndHostPtr);       \
    if (xValHostPtr)        free(xValHostPtr);       \
    if (cooRowIndexHostPtr) free(cooRowIndexHostPtr);\
    if (cooColIndexHostPtr) free(cooColIndexHostPtr);\
    if (cooValHostPtr)      free(cooValHostPtr);     \
    if (y)                  cudaFree(y);             \
    if (z)                  cudaFree(z);             \
    if (xInd)               cudaFree(xInd);          \
    if (xVal)               cudaFree(xVal);          \
    if (csrRowPtr)          cudaFree(csrRowPtr);     \
    if (cooRowIndex)        cudaFree(cooRowIndex);   \
    if (cooColIndex)        cudaFree(cooColIndex);   \
    if (cooVal)             cudaFree(cooVal);        \
    if (handle)             cusparseDestroy(handle); \
    fflush (stdout);                                 \
} while (0)

int main(){     
    cudaError_t cudaStat1,cudaStat2,cudaStat3,cudaStat4,cudaStat5,cudaStat6;
    cusparseStatus_t status;
    cusparseHandle_t handle=0;
    cusparseMatDescr_t descra=0;
    int *    cooRowIndexHostPtr=0;
    int *    cooColIndexHostPtr=0;    
    double * cooValHostPtr=0;
    int *    cooRowIndex=0;
    int *    cooColIndex=0;    
    double * cooVal=0;
    int *    xIndHostPtr=0;
    double * xValHostPtr=0;
    double * yHostPtr=0;
    int *    xInd=0;
    double * xVal=0;
    double * y=0;  
    int *    csrRowPtr=0;
    double * zHostPtr=0; 
    double * z=0; 
    int      n, nnz, nnz_vector, i, j;
    printf("testing example\n");
    /* create the following sparse test matrix in COO format */
    /* |1.0     2.0 3.0|
       |    4.0        |
       |5.0     6.0 7.0|
       |    8.0     9.0| */
    n=4; nnz=9; 
    cooRowIndexHostPtr = (int *)   malloc(nnz*sizeof(cooRowIndexHostPtr[0])); 
    cooColIndexHostPtr = (int *)   malloc(nnz*sizeof(cooColIndexHostPtr[0])); 
    cooValHostPtr      = (double *)malloc(nnz*sizeof(cooValHostPtr[0])); 
    if ((!cooRowIndexHostPtr) || (!cooColIndexHostPtr) || (!cooValHostPtr)){
        CLEANUP("Host malloc failed (matrix)");
        return EXIT_FAILURE; 
    }
    cooRowIndexHostPtr[0]=0; cooColIndexHostPtr[0]=0; cooValHostPtr[0]=1.0;  
    cooRowIndexHostPtr[1]=0; cooColIndexHostPtr[1]=2; cooValHostPtr[1]=2.0;  
    cooRowIndexHostPtr[2]=0; cooColIndexHostPtr[2]=3; cooValHostPtr[2]=3.0;  
    cooRowIndexHostPtr[3]=1; cooColIndexHostPtr[3]=1; cooValHostPtr[3]=4.0;  
    cooRowIndexHostPtr[4]=2; cooColIndexHostPtr[4]=0; cooValHostPtr[4]=5.0;  
    cooRowIndexHostPtr[5]=2; cooColIndexHostPtr[5]=2; cooValHostPtr[5]=6.0;
    cooRowIndexHostPtr[6]=2; cooColIndexHostPtr[6]=3; cooValHostPtr[6]=7.0;  
    cooRowIndexHostPtr[7]=3; cooColIndexHostPtr[7]=1; cooValHostPtr[7]=8.0;  
    cooRowIndexHostPtr[8]=3; cooColIndexHostPtr[8]=3; cooValHostPtr[8]=9.0;  

    //print the matrix
    printf("Input data:\n");
    for (i=0; i<nnz; i++){        
      printf("(%d, %d, %f)\n", cooRowIndexHostPtr[i], cooColIndexHostPtr[i] ,cooValHostPtr[i]);
    }
  
    /* create a sparse and dense vector */ 
    /* xVal= [100.0 200.0 400.0]   (sparse)
       xInd= [0     1     3    ]
       y   = [10.0 20.0 30.0 40.0 | 50.0 60.0 70.0 80.0] (dense) */
    nnz_vector = 3;
    xIndHostPtr = (int *)   malloc(nnz_vector*sizeof(xIndHostPtr[0])); 
    xValHostPtr = (double *)malloc(nnz_vector*sizeof(xValHostPtr[0])); 
    yHostPtr    = (double *)malloc(2*n       *sizeof(yHostPtr[0]));
    zHostPtr    = (double *)malloc(2*(n+1)   *sizeof(zHostPtr[0]));
    if((!xIndHostPtr) || (!xValHostPtr) || (!yHostPtr) || (!zHostPtr)){
        CLEANUP("Host malloc failed (vectors)");
        return EXIT_FAILURE; 
    }


    yHostPtr[0] = 10.0;  xIndHostPtr[0]=0; xValHostPtr[0]=100.0; 
    yHostPtr[1] = 20.0;  xIndHostPtr[1]=1; xValHostPtr[1]=200.0;  
    yHostPtr[2] = 30.0;
    yHostPtr[3] = 40.0;  xIndHostPtr[2]=3; xValHostPtr[2]=400.0;  
    yHostPtr[4] = 100.0;
    yHostPtr[5] = 200.0;
    yHostPtr[6] = 70.0;
    yHostPtr[7] = 400.0;
    //print the vectors
    for (j=0; j<2; j++){
        for (i=0; i<n; i++){        
            printf("yHostPtr[%d,%d]=%f\n",i,j,yHostPtr[i+n*j]);
        }
    }
    for (i=0; i<nnz_vector; i++){        
        printf("xIndHostPtr[%d]=%d  ",i,xIndHostPtr[i]);

        printf("xValHostPtr[%d]=%f\n",i,xValHostPtr[i]);
    }
    /* allocate GPU memory and copy the matrix and vectors into it */
    cudaStat1 = cudaMalloc((void**)&cooRowIndex,nnz*sizeof(cooRowIndex[0])); 
    cudaStat2 = cudaMalloc((void**)&cooColIndex,nnz*sizeof(cooColIndex[0]));
    cudaStat3 = cudaMalloc((void**)&cooVal,     nnz*sizeof(cooVal[0])); 
    cudaStat4 = cudaMalloc((void**)&y,          2*n*sizeof(y[0]));   
    cudaStat5 = cudaMalloc((void**)&xInd,nnz_vector*sizeof(xInd[0])); 
    cudaStat6 = cudaMalloc((void**)&xVal,nnz_vector*sizeof(xVal[0])); 
    if ((cudaStat1 != cudaSuccess) ||
        (cudaStat2 != cudaSuccess) ||
        (cudaStat3 != cudaSuccess) ||
        (cudaStat4 != cudaSuccess) ||
        (cudaStat5 != cudaSuccess) ||
        (cudaStat6 != cudaSuccess)) {
        CLEANUP("Device malloc failed");
        return EXIT_FAILURE; 
    }    
    cudaStat1 = cudaMemcpy(cooRowIndex, cooRowIndexHostPtr, 
                           (size_t)(nnz*sizeof(cooRowIndex[0])), 
                           cudaMemcpyHostToDevice);
    cudaStat2 = cudaMemcpy(cooColIndex, cooColIndexHostPtr, 
                           (size_t)(nnz*sizeof(cooColIndex[0])), 
                           cudaMemcpyHostToDevice);
    cudaStat3 = cudaMemcpy(cooVal,      cooValHostPtr,      
                           (size_t)(nnz*sizeof(cooVal[0])),      
                           cudaMemcpyHostToDevice);
    cudaStat4 = cudaMemcpy(y,           yHostPtr,           
                           (size_t)(2*n*sizeof(y[0])),           
                           cudaMemcpyHostToDevice);
    cudaStat5 = cudaMemcpy(xInd,        xIndHostPtr,        
                           (size_t)(nnz_vector*sizeof(xInd[0])), 
                           cudaMemcpyHostToDevice);
    cudaStat6 = cudaMemcpy(xVal,        xValHostPtr,        
                           (size_t)(nnz_vector*sizeof(xVal[0])), 
                           cudaMemcpyHostToDevice);

    if ((cudaStat1 != cudaSuccess) ||
        (cudaStat2 != cudaSuccess) ||
        (cudaStat3 != cudaSuccess) ||
        (cudaStat4 != cudaSuccess) ||
        (cudaStat5 != cudaSuccess) ||
        (cudaStat6 != cudaSuccess)) {
        CLEANUP("Memcpy from Host to Device failed");
        return EXIT_FAILURE;
    }
    
    /* initialize cusparse library */
    status= cusparseCreate(&handle);
    if (status != CUSPARSE_STATUS_SUCCESS) {
        CLEANUP("CUSPARSE Library initialization failed");
        return EXIT_FAILURE;
    }
    /* create and setup matrix descriptor */ 
    status= cusparseCreateMatDescr(&descra); 
    if (status != CUSPARSE_STATUS_SUCCESS) {
        CLEANUP("Matrix descriptor initialization failed");
        return EXIT_FAILURE;
    }       
    cusparseSetMatType(descra,CUSPARSE_MATRIX_TYPE_GENERAL);
    cusparseSetMatIndexBase(descra,CUSPARSE_INDEX_BASE_ZERO);  
    
    /* exercise conversion routines (convert matrix from COO 2 CSR format) */
    cudaStat1 = cudaMalloc((void**)&csrRowPtr,(n+1)*sizeof(csrRowPtr[0]));
    if (cudaStat1 != cudaSuccess) {
        CLEANUP("Device malloc failed (csrRowPtr)");
        return EXIT_FAILURE;
    }

    status= cusparseXcoo2csr(handle,cooRowIndex,nnz,n,
                             csrRowPtr,CUSPARSE_INDEX_BASE_ZERO); 
    if (status != CUSPARSE_STATUS_SUCCESS) {
        CLEANUP("Conversion from COO to CSR format failed");
        return EXIT_FAILURE;
    }  

    print_ints_gpu(n+1, csrRowPtr);

    //csrRowPtr = [0 3 4 7 9] 
    /* exercise Level 1 routines (scatter vector elements) */
    //status= cusparseDsctr(handle, nnz_vector, xVal, xInd, 
    //                      &y[n], CUSPARSE_INDEX_BASE_ZERO);
    //if (status != CUSPARSE_STATUS_SUCCESS) {
    //    CLEANUP("Scatter from sparse to dense vector failed");
    //    return EXIT_FAILURE;
    //}  

    //y = [10 20 30 40 | 100 200 70 400]
    /* exercise Level 2 routines (csrmv) */
    status= cusparseDcsrmv(handle,CUSPARSE_OPERATION_NON_TRANSPOSE, n, n, 2.0,
                           descra, cooVal, csrRowPtr, cooColIndex, &y[0],
                           3.0, &y[n]);
    if (status != CUSPARSE_STATUS_SUCCESS) {
        CLEANUP("Matrix‐vector multiplication failed");
        return EXIT_FAILURE;
    }  
    
    /* print intermediate results (y) */
    //y = [10 20 30 40 | 680 760 1230 2240]
    cudaMemcpy(yHostPtr, y, (size_t)(2*n*sizeof(y[0])), cudaMemcpyDeviceToHost);
    printf("Intermediate results:\n");
    for (j=0; j<2; j++){
        for (i=0; i<n; i++){        
            printf("yHostPtr[%d,%d]=%f\n",i,j,yHostPtr[i+n*j]);
        }
    }
    
    /* check the results */
    /* Notice that CLEANUP() contains a call to cusparseDestroy(handle) */
    if ((yHostPtr[0] != 10.0)     || 
        (yHostPtr[1] != 20.0)     || 
        (yHostPtr[2] != 30.0)     || 
        (yHostPtr[3] != 40.0)     || 
        (yHostPtr[4] != 680.0)    || 
        (yHostPtr[5] != 760.0)    || 
        (yHostPtr[6] != 1230.0)   || 
        (yHostPtr[7] != 2240.0)){ 
        CLEANUP("example test FAILED");
        return EXIT_FAILURE;
    }
    else{
        CLEANUP("example test PASSED");
        return EXIT_SUCCESS;
    }      
}
