#include "../src/set.h"

#include <stdlib.h>
#include <stdbool.h>
#include <stdio.h>
#include <time.h>
#include <sys/time.h>

#include <cuda_runtime.h>

#include <bits/timing.h>

/**
 * @note Caller must call @c cudaFree on result
 */
bool *random_set_gpu(int n, int ntrue)
{
  bool *cpu, *gpu;
  int i;

  cpu = malloc(n * sizeof(bool));
  for(i = 0; i < n; i++) {
    cpu[i] = false;
  }

  for(i = 0; i < ntrue; i++) {
    cpu[rand() % n] = true;
  }

  cudaMalloc((void **)&gpu, n * sizeof(bool));
  cudaMemcpy(gpu, cpu, n * sizeof(bool), cudaMemcpyHostToDevice);

  free(cpu);

  return gpu;
}

int main(int argc, char **argv)
{
  int n, i, reps;
  bool *S, *T, *work, *host_work, result;

  int device;
  struct cudaDeviceProp devprops;

  struct timeval timer;
  double duration;

  reps = 10;
  n = 128*128;

  srand(time(NULL));
  duration = 0;

  // Get device properties
  cudaGetDevice(&device);
  cudaGetDeviceProperties(&devprops, device);

  // Allocate work vectors
  cudaMalloc((void **)&work, 3*n*sizeof(bool));
  host_work = (bool *)malloc(n * sizeof(bool));

  T = random_set_gpu(n, 0);

  for(i = 0; i < reps; i++) {
    S = random_set_gpu(n, 3);

    //result = set_or(n, S, work, host_work, &devprops);
    //printf("%s\n", result ? "true" : "false");
    cudaMemcpy(T, S, n*sizeof(bool), cudaMemcpyDeviceToDevice);

    timer_start(&timer);
    result = sets_equal(n, S, T, work, host_work, &devprops);
    duration += timer_interval(&timer);
    printf("%s\n", result ? "true" : "false");

    cudaFree(S);
  }

  printf("Time per rep: "); time_print(stdout, duration/reps); fputc('\n', stdout);

  cudaFree(T);
  free(host_work);
  cudaFree(work);

  return 0;
}
